@@ -49,6 +49,9 @@ SOFTWARE.
 #endif
 
 #include <X11/X.h>
+
+#include "dix/ptrveloc_priv.h"
+
 #include "misc.h"
 #include "resource.h"
 #include <X11/Xproto.h>
@@ -796,7 +799,7 @@ InitAndStartDevices(void)
 /**
  * Free the given device class and reset the pointer to NULL.
  */
-static void
+void
 FreeDeviceClass(int type, void **class)
 {
     if (!(*class))
@@ -845,6 +848,14 @@ FreeDeviceClass(int type, void **class)
         free((*t));
         break;
     }
+    case XIGestureClass:
+    {
+        GestureClassPtr *g = (GestureClassPtr *) class;
+
+        GestureFreeGestureInfo(&(*g)->gesture);
+        free((*g));
+        break;
+    }
     case FocusClass:
     {
         FocusClassPtr *f = (FocusClassPtr *) class;
@@ -959,6 +970,7 @@ FreeAllDeviceClasses(ClassesPtr classes)
     FreeDeviceClass(ButtonClass, (void *) &classes->button);
     FreeDeviceClass(FocusClass, (void *) &classes->focus);
     FreeDeviceClass(ProximityClass, (void *) &classes->proximity);
+    FreeDeviceClass(XIGestureClass, (void*) &classes->gesture);
 
     FreeFeedbackClass(KbdFeedbackClass, (void *) &classes->kbdfeed);
     FreeFeedbackClass(PtrFeedbackClass, (void *) &classes->ptrfeed);
@@ -1030,7 +1042,7 @@ CloseDevice(DeviceIntPtr dev)
     free(dev->config_info);     /* Allocated in xf86ActivateDevice. */
     free(dev->last.scroll);
     for (j = 0; j < dev->last.num_touches; j++)
-        free(dev->last.touches[j].valuators);
+        valuator_mask_free(&dev->last.touches[j].valuators);
     free(dev->last.touches);
     dev->config_info = NULL;
     dixFreePrivates(dev->devPrivates, PRIVATE_DEVICE);
@@ -1040,8 +1052,7 @@ CloseDevice(DeviceIntPtr dev)
 /**
  * Shut down all devices of one list and free all resources.
  */
-static
-    void
+static void
 CloseDeviceList(DeviceIntPtr *listHead)
 {
     /* Used to mark devices that we tried to free */
@@ -1148,6 +1159,26 @@ UndisplayDevices(void)
         screen->DisplayCursor(dev, screen, NullCursor);
 }
 
+static int
+CloseOneDevice(const DeviceIntPtr dev, DeviceIntPtr *listHead)
+{
+    DeviceIntPtr tmp, next, prev = NULL;
+
+    for (tmp = *listHead; tmp; (prev = tmp), (tmp = next)) {
+        next = tmp->next;
+        if (tmp == dev) {
+            if (prev == NULL)
+                *listHead = next;
+            else
+                prev->next = next;
+
+            CloseDevice(tmp);
+            return Success;
+        }
+    }
+    return BadMatch;
+}
+
 /**
  * Remove a device from the device list, closes it and thus frees all
  * resources.
@@ -1164,12 +1195,12 @@ UndisplayDevices(void)
 int
 RemoveDevice(DeviceIntPtr dev, BOOL sendevent)
 {
-    DeviceIntPtr prev, tmp, next;
     int ret = BadMatch;
     ScreenPtr screen = screenInfo.screens[0];
     int deviceid;
     int initialized;
     int flags[MAXDEVICES] = { 0 };
+    int flag;
 
     DebugF("(dix) removing device %d\n", dev->id);
 
@@ -1187,41 +1218,13 @@ RemoveDevice(DeviceIntPtr dev, BOOL sendevent)
         flags[dev->id] = XIDeviceDisabled;
     }
 
+    flag = IsMaster(dev) ? XIMasterRemoved : XISlaveRemoved;
+
     input_lock();
 
-    prev = NULL;
-    for (tmp = inputInfo.devices; tmp; (prev = tmp), (tmp = next)) {
-        next = tmp->next;
-        if (tmp == dev) {
-
-            if (prev == NULL)
-                inputInfo.devices = next;
-            else
-                prev->next = next;
-
-            flags[tmp->id] = IsMaster(tmp) ? XIMasterRemoved : XISlaveRemoved;
-            CloseDevice(tmp);
-            ret = Success;
-            break;
-        }
-    }
-
-    prev = NULL;
-    for (tmp = inputInfo.off_devices; tmp; (prev = tmp), (tmp = next)) {
-        next = tmp->next;
-        if (tmp == dev) {
-            flags[tmp->id] = IsMaster(tmp) ? XIMasterRemoved : XISlaveRemoved;
-            CloseDevice(tmp);
-
-            if (prev == NULL)
-                inputInfo.off_devices = next;
-            else
-                prev->next = next;
-
-            ret = Success;
-            break;
-        }
-    }
+    if ((ret = CloseOneDevice(dev, &inputInfo.devices)) == Success ||
+        (ret = CloseOneDevice(dev, &inputInfo.off_devices)) == Success)
+        flags[deviceid] = flag;
 
     input_unlock();
 
