@@ -116,6 +116,8 @@ static uint32_t
 gbm_format_for_depth(int depth, int gles)
 {
     switch (depth) {
+    case 15:
+        return GBM_FORMAT_ARGB1555;
     case 16:
         return GBM_FORMAT_RGB565;
     case 24:
@@ -958,7 +960,7 @@ xwl_glamor_dmabuf_import_sync_file(PixmapPtr pixmap, int sync_file)
 #endif /* GBM_BO_FD_FOR_PLANE */
         struct dma_buf_import_sync_file import_args = { 0 };
         import_args.fd = sync_file;
-        import_args.flags = DMA_BUF_SYNC_READ;
+        import_args.flags = DMA_BUF_SYNC_WRITE;
         drmIoctl(plane_fd, DMA_BUF_IOCTL_IMPORT_SYNC_FILE, &import_args);
         close(plane_fd);
     }
