@@ -37,8 +37,6 @@
 
 #include "xdg-output-unstable-v1-client-protocol.h"
 
-#define MAX_OUTPUT_NAME 256
-
 static void xwl_output_get_xdg_output(struct xwl_output *xwl_output);
 
 static Rotation
@@ -683,10 +681,13 @@ apply_output_change(struct xwl_output *xwl_output)
     maybe_update_fullscreen_state(xwl_output);
 }
 
-static void
+void
 xwl_output_set_name(struct xwl_output *xwl_output, const char *name)
 {
     struct xwl_screen *xwl_screen = xwl_output->xwl_screen;
+    rrScrPrivPtr pScrPriv;
+    RRLeasePtr lease;
+    int i;
 
     if (xwl_output->randr_output == NULL)
         return; /* rootful */
@@ -697,6 +698,24 @@ xwl_output_set_name(struct xwl_output *xwl_output, const char *name)
         return;
     }
 
+    /* Check for duplicate names to be safe */
+    pScrPriv = rrGetScrPriv(xwl_screen->screen);
+    for (i = 0; i < pScrPriv->numOutputs; i++) {
+        if (!strcmp(name, pScrPriv->outputs[i]->name)) {
+            ErrorF("An output named '%s' already exists", name);
+            return;
+        }
+    }
+    /* And leases' names as well */
+    xorg_list_for_each_entry(lease, &pScrPriv->leases, list) {
+        for (i = 0; i < lease->numOutputs; i++) {
+            if (!strcmp(name, pScrPriv->outputs[i]->name)) {
+                ErrorF("A lease output named '%s' already exists", name);
+                return;
+            }
+        }
+    }
+
     snprintf(xwl_output->randr_output->name, MAX_OUTPUT_NAME, "%s", name);
     xwl_output->randr_output->nameLength = strlen(xwl_output->randr_output->name);
 
