@@ -1811,7 +1811,28 @@ xwl_config_notify(WindowPtr window,
     xwl_screen->ConfigNotify = screen->ConfigNotify;
     screen->ConfigNotify = xwl_config_notify;
 
-    if (size_changed && xwl_window) {
+    return ret;
+}
+
+void
+xwl_resize_window(WindowPtr window,
+                  int x, int y,
+                  unsigned int width, unsigned int height,
+                  WindowPtr sib)
+{
+    ScreenPtr screen = window->drawable.pScreen;
+    struct xwl_screen *xwl_screen;
+    struct xwl_window *xwl_window;
+
+    xwl_screen = xwl_screen_get(screen);
+    xwl_window = xwl_window_from_window(window);
+
+    screen->ResizeWindow = xwl_screen->ResizeWindow;
+    screen->ResizeWindow(window, x, y, width, height, sib);
+    xwl_screen->ResizeWindow = screen->ResizeWindow;
+    screen->ResizeWindow = xwl_resize_window;
+
+    if (xwl_window) {
         if (xwl_window_get(window) || xwl_window_is_toplevel(window))
             xwl_window_check_resolution_change_emulation(xwl_window);
         if (window == screen->root) {
@@ -1826,8 +1847,6 @@ xwl_config_notify(WindowPtr window,
             xwl_window_check_fractional_scale_viewport(xwl_window, width, height);
         }
     }
-
-    return ret;
 }
 
 void
